package com.fastcashier.lib_common.function.print

import android.text.TextUtils
import com.fastcashier.lib_common.function.account.AccountService
import com.fastcashier.lib_common.function.account.SelfCashierAccountInfo
import com.fastcashier.lib_common.function.pay.MiYaPayPlantformPayWayEnum
import com.fastcashier.lib_common.function.pay.OrderRecordManageKit
import com.fastcashier.lib_common.util.DateUtils
import com.fastcashier.lib_common.util.StringPriceFormatUtils
import com.fastcashier.lib_common.util.isEmpty
import com.miya.print.IPrinter
import com.miya.print.PrinterManager
import com.miya.print.utils.Page58MmPrintUtils
import com.sdy.miya.moblie.component.pay.platform.bean.PayServiceResponse
import java.util.*
import kotlin.concurrent.schedule

object PrintService {

    private var payServiceResponse: PayServiceResponse? = null

    /**
     * 打印上一单
     */
    fun printLastOrderInfo() {
        if (payServiceResponse == null) {
            throw java.lang.RuntimeException("暂无上一单信息")
        } else {
            AccountService.getAccountInfo()?.let { payPrint(payServiceResponse!!, it) }
        }
    }

    /**
     * 收费打印
     */
    fun payPrint(
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo
    ) {
        //保存上一单信息
        this.payServiceResponse = payServiceResponse

        //记录流水号，退款要用
        val serialNumber = OrderRecordManageKit.getSerialNumber()
        OrderRecordManageKit.setSerialNoByOrderNo(payServiceResponse.shopTradeNo, serialNumber)

        val printer = PrinterManager.getInstance().printer
        printer.beginPrint()
        //检查打印机状态
        printer.printerStatus
        printer.set58mm()
        handlePayPrint(printer, payServiceResponse, selfCashierAccountInfo, serialNumber)
        printer.feedPaper()
        printer.endPrint()
    }

    private fun handlePayPrint(
        printer: IPrinter,
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo,
        serialNumber: String
    ) {
//        if (BuildConfig.DEBUG) {
//            printCashier(printer, payServiceResponse, selfCashierAccountInfo, serialNumber)
//        } else {
        printCashier(
            printer,
            payServiceResponse,
            selfCashierAccountInfo,
            serialNumber
        )

        Timer().schedule(4000) {
            printCustomer(
                printer,
                payServiceResponse,
                selfCashierAccountInfo,
                serialNumber
            )
        }
//        }
    }

    /**
     * 打印收银员联
     */
    private fun printCashier(
        printer: IPrinter,
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo,
        serialNumber: String
    ) {
        val shopInfo = selfCashierAccountInfo.shopInfo
        printer.printText(IPrinter.ALIGN_CENTER, 3, shopInfo.storeName, true)
        printer.setAlign(IPrinter.ALIGN_LEFT)
        printer.printText("商户编号：${shopInfo.saasid}")
        printer.printText("终端编号：${shopInfo.posId}")
        printer.printText("商户名称：converse")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("交易类型：")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    ${MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc}",
            true
        )
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("日期/时间：${DateUtils.format18(Date())}")
        printer.printText("流水号：${serialNumber}")
        printer.printText("商户订单号：")
        printer.printText(payServiceResponse.shopTradeNo)
        printer.printText("支付平台交易号：")
        printer.printText(payServiceResponse.chanelSerialNumber)
        printer.printText("金额:")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    RMB${StringPriceFormatUtils.transStringPriceToDecimalString(payServiceResponse.tradPrice)}",
            true
        )
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("备注：")

        val couponDetail: Array<String> = payServiceResponse.transformCouponMessage
        if (couponDetail.size >= 2 && !TextUtils.isEmpty(couponDetail[1]) && couponDetail[1] != "0" && couponDetail[1] != "0.00") {
            val shopCoupon = couponDetail[1]
            //打印商家优惠
            printer.printText("商家优惠：${StringPriceFormatUtils.transStringPriceToDecimalString(shopCoupon)}元")
        }
        if (couponDetail.size >= 3 && !TextUtils.isEmpty(couponDetail[2]) && couponDetail[2] != "0" && couponDetail[2] != "0.00") {
            //平台优惠
            val platformCoupon = couponDetail[2]
            val channel =
                MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc
            printer.printText(
                "$channel 优惠：${
                    StringPriceFormatUtils.transStringPriceToDecimalString(
                        platformCoupon
                    )
                }元"
            )
        }
        if (couponDetail.isNotEmpty() && !TextUtils.isEmpty(couponDetail[0])) {
            val payPrice = couponDetail[0]
            printer.printText("客户支付：${StringPriceFormatUtils.transStringPriceToDecimalString(payPrice)}元")
        }

        printer.printText("持卡人签名：")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("同意以上支付 ")
        printer.printText("\n\n")
    }

    /**
     * 打印顾客联
     */
    private fun printCustomer(
        printer: IPrinter,
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo,
        serialNumber: String
    ) {
        val shopInfo = selfCashierAccountInfo.shopInfo
        printer.printText(IPrinter.ALIGN_CENTER, 3, shopInfo.storeName, true)
        printer.setAlign(IPrinter.ALIGN_LEFT)
        printer.printText("持卡人存根")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("商户编号：${shopInfo.saasid}")
        printer.printText("终端编号：${shopInfo.posId}")
        printer.printText("商户名称：converse")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("交易类型：")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    ${MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc}",
            true
        )
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("日期/时间：${DateUtils.format18(Date())}")
        printer.printText("流水号：${serialNumber}")
        printer.printText("商户订单号：")
        printer.printText(payServiceResponse.shopTradeNo)
        printer.printText("支付平台交易号：")
        printer.printText(payServiceResponse.chanelSerialNumber)
        printer.printText("金额:")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    RMB${StringPriceFormatUtils.transStringPriceToDecimalString(payServiceResponse.tradPrice)}",
            true
        )
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("备注：")

        val couponDetail: Array<String> = payServiceResponse.transformCouponMessage
        if (couponDetail.size >= 2 && !TextUtils.isEmpty(couponDetail[1]) && couponDetail[1] != "0" && couponDetail[1] != "0.00") {
            val shopCoupon = couponDetail[1]
            //打印商家优惠
            printer.printText("商家优惠：${StringPriceFormatUtils.transStringPriceToDecimalString(shopCoupon)}元")
        }
        if (couponDetail.size >= 3 && !TextUtils.isEmpty(couponDetail[2]) && couponDetail[2] != "0" && couponDetail[2] != "0.00") {
            //平台优惠
            val platformCoupon = couponDetail[2]
            val channel =
                MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc
            printer.printText(
                "$channel 优惠：${
                    StringPriceFormatUtils.transStringPriceToDecimalString(
                        platformCoupon
                    )
                }元"
            )
        }
        if (couponDetail.isNotEmpty() && !TextUtils.isEmpty(couponDetail[0])) {
            val payPrice = couponDetail[0]
            printer.printText("客户支付：${StringPriceFormatUtils.transStringPriceToDecimalString(payPrice)}元")
        }
        printer.printText("\n\n")
    }


    /**
     * 查询订单单项打印
     */
    fun printSearchOrder(
        selfCashierAccountInfo: SelfCashierAccountInfo,
        payServiceResponse: PayServiceResponse,
        isRefund: Boolean
    ) {

        if (isRefund) {
            refundPrint(selfCashierAccountInfo, payServiceResponse)
        } else {
            val printer = PrinterManager.getInstance().printer
            printer.beginPrint()
            //检查打印机状态
            printer.printerStatus
            printer.set58mm()
            handlePayPrint(
                printer,
                payServiceResponse,
                selfCashierAccountInfo,
                OrderRecordManageKit.getSerialNoByOrderNo(payServiceResponse.shopTradeNo)
            )
            printer.feedPaper()
            printer.endPrint()
        }
    }

    /**
     * 退款
     */
    fun refundPrint(
        selfCashierAccountInfo: SelfCashierAccountInfo,
        payServiceResponse: PayServiceResponse
    ) {
        val printer = PrinterManager.getInstance().printer
        printer.beginPrint()
        //检查打印机状态
        printer.printerStatus
        printer.set58mm()
        handleRefundPrint(printer, payServiceResponse, selfCashierAccountInfo)
        printer.feedPaper()
        printer.endPrint()
    }

    fun handleRefundPrint(
        printer: IPrinter,
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo
    ) {
        refundPrintCashier(
            printer,
            payServiceResponse,
            selfCashierAccountInfo
        )
        Timer().schedule(4000) {
            refundPrintCustomer(
                printer,
                payServiceResponse,
                selfCashierAccountInfo
            )
        }
    }

    /**
     * 打印收银员联 - 退款
     */
    private fun refundPrintCashier(
        printer: IPrinter,
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo
    ) {
        val shopInfo = selfCashierAccountInfo.shopInfo
        printer.printText(IPrinter.ALIGN_CENTER, 3, shopInfo.storeName, true)
        printer.setAlign(IPrinter.ALIGN_LEFT)
        printer.printText("商户编号：${shopInfo.saasid}")
        printer.printText("终端编号：${shopInfo.posId}")
        printer.printText("商户名称：converse")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("交易类型：")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    退款(${MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc})",
            true
        )
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("日期/时间：${DateUtils.format18(Date())}")
        printer.printText("流水号：${OrderRecordManageKit.getSerialNoByOrderNo(payServiceResponse.shopTradeNo)}")
        printer.printText("商户订单号：")
        printer.printText(payServiceResponse.shopTradeNo)
        printer.printText("支付平台交易号：")
        printer.printText(payServiceResponse.chanelSerialNumber)
        printer.printText("金额:")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    RMB${StringPriceFormatUtils.transStringPriceToDecimalString(payServiceResponse.refundPrice)}",
            true
        )
        printer.printText("原订单号：")
        printer.printText(payServiceResponse.shopTradeNo)
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("备注：")

        val couponDetail: Array<String> = payServiceResponse.transformCouponMessage
        if (couponDetail.size >= 2 && !TextUtils.isEmpty(couponDetail[1]) && couponDetail[1] != "0" && couponDetail[1] != "0.00") {
            val shopCoupon = couponDetail[1]
            //打印商家优惠
            printer.printText("商家优惠：${StringPriceFormatUtils.transStringPriceToDecimalString(shopCoupon)}元")
        }
        if (couponDetail.size >= 3 && !TextUtils.isEmpty(couponDetail[2]) && couponDetail[2] != "0" && couponDetail[2] != "0.00") {
            //平台优惠
            val platformCoupon = couponDetail[2]
            val channel =
                MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc
            printer.printText(
                "$channel 优惠：${
                    StringPriceFormatUtils.transStringPriceToDecimalString(
                        platformCoupon
                    )
                }元"
            )
        }
        if (couponDetail.isNotEmpty() && !TextUtils.isEmpty(couponDetail[0])) {
            val payPrice = couponDetail[0]
            printer.printText("客户支付：${StringPriceFormatUtils.transStringPriceToDecimalString(payPrice)}元")
        }

        printer.printText("持卡人签名：")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("同意以上支付 ")
        printer.printText("\n\n")
    }

    /**
     * 打印顾客联-退款
     */
    private fun refundPrintCustomer(
        printer: IPrinter,
        payServiceResponse: PayServiceResponse,
        selfCashierAccountInfo: SelfCashierAccountInfo
    ) {
        val shopInfo = selfCashierAccountInfo.shopInfo
        printer.printText(IPrinter.ALIGN_CENTER, 3, shopInfo.storeName, true)
        printer.setAlign(IPrinter.ALIGN_LEFT)
        printer.printText("持卡人存根")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("商户编号：${shopInfo.saasid}")
        printer.printText("终端编号：${shopInfo.posId}")
        printer.printText("商户名称：converse")
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("交易类型：")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    退款(${MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc})",
            true
        )
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("日期/时间：${DateUtils.format18(Date())}")
        printer.printText("流水号：${OrderRecordManageKit.getSerialNoByOrderNo(payServiceResponse.shopTradeNo)}")
        printer.printText("商户订单号：")
        printer.printText(payServiceResponse.refundOrderNo)
        printer.printText("支付平台交易号：")
        printer.printText(payServiceResponse.chanelSerialNumber)
        printer.printText("金额:")
        printer.printText(
            IPrinter.ALIGN_LEFT,
            3,
            "    RMB${StringPriceFormatUtils.transStringPriceToDecimalString(payServiceResponse.refundPrice)}",
            true
        )
        printer.printText("原订单号：")
        printer.printText(payServiceResponse.shopTradeNo)
        printer.printText(Page58MmPrintUtils.printDivideLineString())
        printer.printText("备注：")

        val couponDetail: Array<String> = payServiceResponse.transformCouponMessage
        if (couponDetail.size >= 2 && !TextUtils.isEmpty(couponDetail[1]) && couponDetail[1] != "0" && couponDetail[1] != "0.00") {
            val shopCoupon = couponDetail[1]
            //打印商家优惠
            printer.printText("商家优惠：${StringPriceFormatUtils.transStringPriceToDecimalString(shopCoupon)}元")
        }
        if (couponDetail.size >= 3 && !TextUtils.isEmpty(couponDetail[2]) && couponDetail[2] != "0" && couponDetail[2] != "0.00") {
            //平台优惠
            val platformCoupon = couponDetail[2]
            val channel =
                MiYaPayPlantformPayWayEnum.MiyaPayType.find(payServiceResponse.chanelTag).desc
            printer.printText(
                "$channel 优惠：${
                    StringPriceFormatUtils.transStringPriceToDecimalString(
                        platformCoupon
                    )
                }元"
            )
        }
        if (couponDetail.isNotEmpty() && !TextUtils.isEmpty(couponDetail[0])) {
            val payPrice = couponDetail[0]
            printer.printText("客户支付：${StringPriceFormatUtils.transStringPriceToDecimalString(payPrice)}元")
        }
        printer.printText("\n\n")
    }


    /**
     * 统计、结算打印,需要起止时间
     */
    fun printStatisticInfo(
        viewOrderStatisticsInfo: ViewOrderStatisticsInfo,
        selfCashierAccountInfo: SelfCashierAccountInfo,
        isBalance: Boolean
    ) {
        val printer = PrinterManager.getInstance().printer
        printer.beginPrint()
        //检查打印机状态
        printer.printerStatus
        printer.set58mm()
        doPrintStatisticInfo(printer, viewOrderStatisticsInfo, selfCashierAccountInfo, isBalance)
        printer.feedPaper()
        printer.endPrint()
    }

    private fun doPrintStatisticInfo(
        printer: IPrinter,
        orderStatisticsInfo: ViewOrderStatisticsInfo,
        accountInfo: SelfCashierAccountInfo,
        isBalance: Boolean
    ) {

        if (accountInfo.shopInfo == null) {
            throw RuntimeException("无商户配置数据")
        }
        val shopInfo = accountInfo.shopInfo
        printer.printText(IPrinter.ALIGN_CENTER, 3, accountInfo.shopInfo?.storeName ?: "", true)
        if (isBalance) {
            printer.printText(IPrinter.ALIGN_CENTER, 3, "结算总计单", true)
        } else {
            printer.printText(IPrinter.ALIGN_CENTER, 3, "交易汇总表", true)
        }

        printer.setAlign(IPrinter.ALIGN_LEFT)
        printer.printText("商户名称：converse")
        printer.printText("商户编号：${shopInfo.saasid}")
        printer.printText("终端编号：${shopInfo.posId}")
        printer.printText("门店编号：${shopInfo.storeId}")

        //统计时间
        printer.printText(0, 0, "开始时间：" + orderStatisticsInfo.beginDate, true)
        printer.printText(0, 0, "结束时间：" + orderStatisticsInfo.endDate, true)

        val totalStatistics: ViewOrderStatisticsInfo.StatisticBean? =
            orderStatisticsInfo.totalStatistic
        if (totalStatistics != null) {
            var tradeCount = totalStatistics.tradeCount + totalStatistics.refundCount

            printer.printText(0, 0, Page58MmPrintUtils.printDivideLineString(), true)
            printer.printText(
                Page58MmPrintUtils.printTwoData(
                    "总交易笔数",
                    "" + tradeCount
                )
            )

            var tradeTotalAmount =
                totalStatistics.tradeTotalAmount - totalStatistics.refundTotalAmount
            printer.printText(
                Page58MmPrintUtils.printTwoData(
                    "总交易金额",
                    "" + StringPriceFormatUtils.transStringPriceToDecimalString(tradeTotalAmount.toString())
                )
            )
        }

        if (!isEmpty(orderStatisticsInfo.typeStatistic)) {
            val typedStatisticList: List<ViewOrderStatisticsInfo.StatisticBean> =
                orderStatisticsInfo.typeStatistic
            var statisticBean: ViewOrderStatisticsInfo.StatisticBean
            for (i in typedStatisticList.indices) {
                if (typedStatisticList[i] == null) {
                    continue
                }
                statisticBean = typedStatisticList[i]
                printer.printText(0, 0, Page58MmPrintUtils.printDivideLineString(), true)
                printer.printText(
                    Page58MmPrintUtils.printTwoData(
                        "支付方式",
                        statisticBean.payType
                    )
                )
                printer.printText(
                    Page58MmPrintUtils.printTwoData(
                        "交易笔数",
                        "" + statisticBean.tradeCount
                    )
                )
                printer.printText(
                    Page58MmPrintUtils.printTwoData(
                        "交易金额",
                        "" + StringPriceFormatUtils.transStringPriceToDecimalString(statisticBean.tradeTotalAmount.toString())
                    )
                )

                if (statisticBean.refundCount > 0) {
                    printer.printText(
                        Page58MmPrintUtils.printTwoData(
                            "退款笔数",
                            "" + statisticBean.refundCount
                        )
                    )
                }

                if (statisticBean.refundTotalAmount > 0) {
                    printer.printText(
                        Page58MmPrintUtils.printTwoData(
                            "退款金额",
                            "" + StringPriceFormatUtils.transStringPriceToDecimalString(statisticBean.refundTotalAmount.toString())
                        )
                    )
                }
            }
        }

        printer.printText("\n\n")
    }
}